precision mediump float;
uniform sampler2D inputImageTexture;
varying vec2 textureCoordinate;

const lowp int GAUSSIAN_SAMPLES = 9;

vec2 blurCoordinates[GAUSSIAN_SAMPLES];
uniform bool isMirror;//是否镜像
uniform bool isMirrorHorizontalOnly;//是否仅显示横向镜像（优先级低于 isMirror）

//高斯模糊需要设置的值
uniform float texelWidthOffset;
uniform float texelHeightOffset;

//径向模糊需要
uniform float radialBlurSize;
uniform highp float radialBlurTextureStepWidth;
uniform highp float radialBlurTextureStepHeight;
const vec2 radialBlurCenter = vec2(0.5, 0.5);

//缩放的大小
uniform float scaleValue;
uniform float scaleAnchorX;//缩放中心点 x
uniform float scaleAnchorY;//缩放中心点 y
//x 轴移动的大小
uniform float transXValue;
//y 轴移动的大小
uniform float transYValue;
//旋转的的大小
uniform float rotate;

uniform bool showMask;//是否显示蒙版
uniform vec4 maskColor;//蒙版的颜色

vec2 scale(vec2 newTextureCoordinate, float scaleValue){
    newTextureCoordinate -= vec2(scaleAnchorX, scaleAnchorY);
    newTextureCoordinate = mat2(scaleValue, 0.0, 0.0, scaleValue) * newTextureCoordinate;
    newTextureCoordinate += vec2(scaleAnchorX, scaleAnchorY);
    return newTextureCoordinate;
}

vec2 rotate2dCenter(vec2 newTextureCoordinate, float angle){
    newTextureCoordinate -= vec2(0.5);
    // rotate the space
    newTextureCoordinate = mat2(cos(angle), -sin(angle), sin(angle), cos(angle)) * newTextureCoordinate;
    // move it back to the original place
    newTextureCoordinate += vec2(0.5);
    return newTextureCoordinate;
}

//获取高斯模糊后的颜色
vec4 getGaussBlurColor(vec2 newTextureCoordinate){
    lowp vec3 sum = vec3(0.0);
    // Calculate the positions for the blur
    int multiplier = 0;
    vec2 blurStep;
    vec2 singleStepOffset = vec2(texelHeightOffset, texelWidthOffset);

    for (int i = 0; i < GAUSSIAN_SAMPLES; i++)
    {
        multiplier = (i - ((GAUSSIAN_SAMPLES - 1) / 2));
        // Blur in x (horizontal)
        blurStep = float(multiplier) * singleStepOffset;
        blurCoordinates[i] = newTextureCoordinate.xy + blurStep;
    }

    lowp vec4 fragColor=texture2D(inputImageTexture, newTextureCoordinate);

    sum += texture2D(inputImageTexture, blurCoordinates[0]).rgb * 0.05;
    sum += texture2D(inputImageTexture, blurCoordinates[1]).rgb * 0.09;
    sum += texture2D(inputImageTexture, blurCoordinates[2]).rgb * 0.12;
    sum += texture2D(inputImageTexture, blurCoordinates[3]).rgb * 0.15;
    sum += texture2D(inputImageTexture, blurCoordinates[4]).rgb * 0.18;
    sum += texture2D(inputImageTexture, blurCoordinates[5]).rgb * 0.15;
    sum += texture2D(inputImageTexture, blurCoordinates[6]).rgb * 0.12;
    sum += texture2D(inputImageTexture, blurCoordinates[7]).rgb * 0.09;
    sum += texture2D(inputImageTexture, blurCoordinates[8]).rgb * 0.05;
    return vec4(sum, fragColor.a);
}


vec4 getValidColor(vec2 newTextureCoordinate, float weight){
    vec4 fragmentColor;
    if (newTextureCoordinate.x >= 0.0 && newTextureCoordinate.x <= 1.0 && newTextureCoordinate.y >= 0.0 && newTextureCoordinate.y <= 1.0){
        return texture2D(inputImageTexture, newTextureCoordinate) * weight;
    }
    return texture2D(inputImageTexture, textureCoordinate) * weight;
}

//获取径向模糊后的颜色
vec4 getRadialBlurColor(vec2 newTextureCoordinate){
    highp vec2 directionOfCoor = radialBlurCenter - newTextureCoordinate;
    directionOfCoor = directionOfCoor / length(directionOfCoor);
    highp vec2 samplingOffset = directionOfCoor * vec2(radialBlurTextureStepWidth, radialBlurTextureStepHeight) * radialBlurSize;

    highp float progress = 1.0;
    highp float tempProgress = 1.0 - progress;
    highp float weight1 = 0.227027;
    weight1 = weight1 + (1.0-weight1) * tempProgress;
    tempProgress = 1.0 - tempProgress;
    highp float weight2 = 0.1945946 * tempProgress;
    highp float weight3 = 0.1216216 * tempProgress;
    highp float weight4 = 0.054054 * tempProgress;
    highp float weight5 = 0.016216 * tempProgress;
    vec4 fragmentColor;

    fragmentColor = getValidColor(newTextureCoordinate, weight1);
    fragmentColor += getValidColor(newTextureCoordinate + samplingOffset, weight2);
    fragmentColor += getValidColor(newTextureCoordinate + (2.0 * samplingOffset), weight3);
    fragmentColor += getValidColor(newTextureCoordinate + (3.0 * samplingOffset), weight4);
    fragmentColor += getValidColor(newTextureCoordinate + (4.0 * samplingOffset), weight5);
    fragmentColor += getValidColor((newTextureCoordinate - samplingOffset), weight2);
    fragmentColor += getValidColor(newTextureCoordinate - (2.0 * samplingOffset), weight3);
    fragmentColor += getValidColor(newTextureCoordinate - (3.0 * samplingOffset), weight4);
    fragmentColor += getValidColor(newTextureCoordinate - (4.0 * samplingOffset), weight5);
    return fragmentColor;
}

vec4 getFragColorByCondition(vec2 newTextureCoordinate){
    if (texelWidthOffset == 0.0 && texelHeightOffset == 0.0){
        if (radialBlurSize == 0.0 || (radialBlurTextureStepWidth == 0.0 && radialBlurTextureStepHeight == 0.0)){
            return texture2D(inputImageTexture, newTextureCoordinate);
        } else {
            //径向模糊
            return getRadialBlurColor(newTextureCoordinate);
        }
    } else {
        //高斯模糊
        return getGaussBlurColor(newTextureCoordinate);
    }
}


void main()
{
    if(showMask){
        gl_FragColor = maskColor;

    }else{
        vec2 newTextureCoordinate = textureCoordinate;
        //先执行偏移
        newTextureCoordinate-=vec2(transXValue, transYValue);
        if (rotate != 0.0){
            newTextureCoordinate = rotate2dCenter(newTextureCoordinate, rotate);
        }
        if (scaleValue != 1.0){
            newTextureCoordinate = scale(newTextureCoordinate, scaleValue);
        }

        if (isMirror || (newTextureCoordinate.y >= 0.0  && newTextureCoordinate.y <= 1.0 && newTextureCoordinate.x >= 0.0  && newTextureCoordinate.x <= 1.0)){
            gl_FragColor = getFragColorByCondition(newTextureCoordinate);
        } else if(isMirrorHorizontalOnly && (newTextureCoordinate.y >= 0.0  && newTextureCoordinate.y <= 1.0)){
            gl_FragColor = getFragColorByCondition(newTextureCoordinate);
        } else {
             gl_FragColor = vec4(0.0);
        }
    }

}
